// Code generated by protoc-gen-gogo. DO NOT EDIT.
// source: fields.proto

//  This file is used for testing discovery of log fields from requests using reflection and gogo proto more tags.

package mwitkow_testproto

import (
	fmt "fmt"
	_ "github.com/gogo/protobuf/gogoproto"
	proto "github.com/golang/protobuf/proto"
	_ "github.com/golang/protobuf/ptypes/timestamp"
	math "math"
	time "time"
)

// Reference imports to suppress errors if they are not otherwise used.
var _ = proto.Marshal
var _ = fmt.Errorf
var _ = math.Inf
var _ = time.Kitchen

// This is a compile-time assertion to ensure that this generated file
// is compatible with the proto package it is being compiled against.
// A compilation error at this line likely means your copy of the
// proto package needs to be updated.
const _ = proto.ProtoPackageIsVersion3 // please upgrade the proto package

type Metadata struct {
	Tags                 []string `protobuf:"bytes,1,rep,name=tags,proto3" json:"tags,omitempty" log_field:"meta_tags"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *Metadata) Reset()         { *m = Metadata{} }
func (m *Metadata) String() string { return proto.CompactTextString(m) }
func (*Metadata) ProtoMessage()    {}
func (*Metadata) Descriptor() ([]byte, []int) {
	return fileDescriptor_d39ad626ec0e575e, []int{0}
}
func (m *Metadata) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_Metadata.Unmarshal(m, b)
}
func (m *Metadata) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_Metadata.Marshal(b, m, deterministic)
}
func (m *Metadata) XXX_Merge(src proto.Message) {
	xxx_messageInfo_Metadata.Merge(m, src)
}
func (m *Metadata) XXX_Size() int {
	return xxx_messageInfo_Metadata.Size(m)
}
func (m *Metadata) XXX_DiscardUnknown() {
	xxx_messageInfo_Metadata.DiscardUnknown(m)
}

var xxx_messageInfo_Metadata proto.InternalMessageInfo

func (m *Metadata) GetTags() []string {
	if m != nil {
		return m.Tags
	}
	return nil
}

type PingId struct {
	Id                   int32    `protobuf:"varint,1,opt,name=id,proto3" json:"id,omitempty" log_field:"ping_id"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *PingId) Reset()         { *m = PingId{} }
func (m *PingId) String() string { return proto.CompactTextString(m) }
func (*PingId) ProtoMessage()    {}
func (*PingId) Descriptor() ([]byte, []int) {
	return fileDescriptor_d39ad626ec0e575e, []int{1}
}
func (m *PingId) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_PingId.Unmarshal(m, b)
}
func (m *PingId) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_PingId.Marshal(b, m, deterministic)
}
func (m *PingId) XXX_Merge(src proto.Message) {
	xxx_messageInfo_PingId.Merge(m, src)
}
func (m *PingId) XXX_Size() int {
	return xxx_messageInfo_PingId.Size(m)
}
func (m *PingId) XXX_DiscardUnknown() {
	xxx_messageInfo_PingId.DiscardUnknown(m)
}

var xxx_messageInfo_PingId proto.InternalMessageInfo

func (m *PingId) GetId() int32 {
	if m != nil {
		return m.Id
	}
	return 0
}

type Ping struct {
	Id                   *PingId  `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	Value                string   `protobuf:"bytes,2,opt,name=value,proto3" json:"value,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *Ping) Reset()         { *m = Ping{} }
func (m *Ping) String() string { return proto.CompactTextString(m) }
func (*Ping) ProtoMessage()    {}
func (*Ping) Descriptor() ([]byte, []int) {
	return fileDescriptor_d39ad626ec0e575e, []int{2}
}
func (m *Ping) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_Ping.Unmarshal(m, b)
}
func (m *Ping) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_Ping.Marshal(b, m, deterministic)
}
func (m *Ping) XXX_Merge(src proto.Message) {
	xxx_messageInfo_Ping.Merge(m, src)
}
func (m *Ping) XXX_Size() int {
	return xxx_messageInfo_Ping.Size(m)
}
func (m *Ping) XXX_DiscardUnknown() {
	xxx_messageInfo_Ping.DiscardUnknown(m)
}

var xxx_messageInfo_Ping proto.InternalMessageInfo

func (m *Ping) GetId() *PingId {
	if m != nil {
		return m.Id
	}
	return nil
}

func (m *Ping) GetValue() string {
	if m != nil {
		return m.Value
	}
	return ""
}

type PingRequest struct {
	Ping                 *Ping     `protobuf:"bytes,1,opt,name=ping,proto3" json:"ping,omitempty"`
	Meta                 *Metadata `protobuf:"bytes,2,opt,name=meta,proto3" json:"meta,omitempty"`
	XXX_NoUnkeyedLiteral struct{}  `json:"-"`
	XXX_unrecognized     []byte    `json:"-"`
	XXX_sizecache        int32     `json:"-"`
}

func (m *PingRequest) Reset()         { *m = PingRequest{} }
func (m *PingRequest) String() string { return proto.CompactTextString(m) }
func (*PingRequest) ProtoMessage()    {}
func (*PingRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_d39ad626ec0e575e, []int{3}
}
func (m *PingRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_PingRequest.Unmarshal(m, b)
}
func (m *PingRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_PingRequest.Marshal(b, m, deterministic)
}
func (m *PingRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_PingRequest.Merge(m, src)
}
func (m *PingRequest) XXX_Size() int {
	return xxx_messageInfo_PingRequest.Size(m)
}
func (m *PingRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_PingRequest.DiscardUnknown(m)
}

var xxx_messageInfo_PingRequest proto.InternalMessageInfo

func (m *PingRequest) GetPing() *Ping {
	if m != nil {
		return m.Ping
	}
	return nil
}

func (m *PingRequest) GetMeta() *Metadata {
	if m != nil {
		return m.Meta
	}
	return nil
}

type Pong struct {
	Id                   string   `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty" log_field:"pong_id"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *Pong) Reset()         { *m = Pong{} }
func (m *Pong) String() string { return proto.CompactTextString(m) }
func (*Pong) ProtoMessage()    {}
func (*Pong) Descriptor() ([]byte, []int) {
	return fileDescriptor_d39ad626ec0e575e, []int{4}
}
func (m *Pong) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_Pong.Unmarshal(m, b)
}
func (m *Pong) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_Pong.Marshal(b, m, deterministic)
}
func (m *Pong) XXX_Merge(src proto.Message) {
	xxx_messageInfo_Pong.Merge(m, src)
}
func (m *Pong) XXX_Size() int {
	return xxx_messageInfo_Pong.Size(m)
}
func (m *Pong) XXX_DiscardUnknown() {
	xxx_messageInfo_Pong.DiscardUnknown(m)
}

var xxx_messageInfo_Pong proto.InternalMessageInfo

func (m *Pong) GetId() string {
	if m != nil {
		return m.Id
	}
	return ""
}

type OneOfLogField struct {
	// Types that are valid to be assigned to Identifier:
	//	*OneOfLogField_BarId
	//	*OneOfLogField_BazId
	Identifier           isOneOfLogField_Identifier `protobuf_oneof:"identifier"`
	XXX_NoUnkeyedLiteral struct{}                   `json:"-"`
	XXX_unrecognized     []byte                     `json:"-"`
	XXX_sizecache        int32                      `json:"-"`
}

func (m *OneOfLogField) Reset()         { *m = OneOfLogField{} }
func (m *OneOfLogField) String() string { return proto.CompactTextString(m) }
func (*OneOfLogField) ProtoMessage()    {}
func (*OneOfLogField) Descriptor() ([]byte, []int) {
	return fileDescriptor_d39ad626ec0e575e, []int{5}
}
func (m *OneOfLogField) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_OneOfLogField.Unmarshal(m, b)
}
func (m *OneOfLogField) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_OneOfLogField.Marshal(b, m, deterministic)
}
func (m *OneOfLogField) XXX_Merge(src proto.Message) {
	xxx_messageInfo_OneOfLogField.Merge(m, src)
}
func (m *OneOfLogField) XXX_Size() int {
	return xxx_messageInfo_OneOfLogField.Size(m)
}
func (m *OneOfLogField) XXX_DiscardUnknown() {
	xxx_messageInfo_OneOfLogField.DiscardUnknown(m)
}

var xxx_messageInfo_OneOfLogField proto.InternalMessageInfo

type isOneOfLogField_Identifier interface {
	isOneOfLogField_Identifier()
}

type OneOfLogField_BarId struct {
	BarId string `protobuf:"bytes,1,opt,name=bar_id,json=barId,proto3,oneof" json:"bar_id,omitempty" log_field:"bar_id"`
}
type OneOfLogField_BazId struct {
	BazId string `protobuf:"bytes,2,opt,name=baz_id,json=bazId,proto3,oneof" json:"baz_id,omitempty" log_field:"baz_id"`
}

func (*OneOfLogField_BarId) isOneOfLogField_Identifier() {}
func (*OneOfLogField_BazId) isOneOfLogField_Identifier() {}

func (m *OneOfLogField) GetIdentifier() isOneOfLogField_Identifier {
	if m != nil {
		return m.Identifier
	}
	return nil
}

func (m *OneOfLogField) GetBarId() string {
	if x, ok := m.GetIdentifier().(*OneOfLogField_BarId); ok {
		return x.BarId
	}
	return ""
}

func (m *OneOfLogField) GetBazId() string {
	if x, ok := m.GetIdentifier().(*OneOfLogField_BazId); ok {
		return x.BazId
	}
	return ""
}

// XXX_OneofWrappers is for the internal use of the proto package.
func (*OneOfLogField) XXX_OneofWrappers() []interface{} {
	return []interface{}{
		(*OneOfLogField_BarId)(nil),
		(*OneOfLogField_BazId)(nil),
	}
}

type PongRequest struct {
	Pong                 *Pong     `protobuf:"bytes,1,opt,name=pong,proto3" json:"pong,omitempty"`
	Meta                 *Metadata `protobuf:"bytes,2,opt,name=meta,proto3" json:"meta,omitempty"`
	XXX_NoUnkeyedLiteral struct{}  `json:"-"`
	XXX_unrecognized     []byte    `json:"-"`
	XXX_sizecache        int32     `json:"-"`
}

func (m *PongRequest) Reset()         { *m = PongRequest{} }
func (m *PongRequest) String() string { return proto.CompactTextString(m) }
func (*PongRequest) ProtoMessage()    {}
func (*PongRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_d39ad626ec0e575e, []int{6}
}
func (m *PongRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_PongRequest.Unmarshal(m, b)
}
func (m *PongRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_PongRequest.Marshal(b, m, deterministic)
}
func (m *PongRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_PongRequest.Merge(m, src)
}
func (m *PongRequest) XXX_Size() int {
	return xxx_messageInfo_PongRequest.Size(m)
}
func (m *PongRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_PongRequest.DiscardUnknown(m)
}

var xxx_messageInfo_PongRequest proto.InternalMessageInfo

func (m *PongRequest) GetPong() *Pong {
	if m != nil {
		return m.Pong
	}
	return nil
}

func (m *PongRequest) GetMeta() *Metadata {
	if m != nil {
		return m.Meta
	}
	return nil
}

type GoGoProtoStdTime struct {
	Timestamp            *time.Time `protobuf:"bytes,1,opt,name=timestamp,proto3,stdtime" json:"timestamp,omitempty"`
	XXX_NoUnkeyedLiteral struct{}   `json:"-"`
	XXX_unrecognized     []byte     `json:"-"`
	XXX_sizecache        int32      `json:"-"`
}

func (m *GoGoProtoStdTime) Reset()         { *m = GoGoProtoStdTime{} }
func (m *GoGoProtoStdTime) String() string { return proto.CompactTextString(m) }
func (*GoGoProtoStdTime) ProtoMessage()    {}
func (*GoGoProtoStdTime) Descriptor() ([]byte, []int) {
	return fileDescriptor_d39ad626ec0e575e, []int{7}
}
func (m *GoGoProtoStdTime) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_GoGoProtoStdTime.Unmarshal(m, b)
}
func (m *GoGoProtoStdTime) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_GoGoProtoStdTime.Marshal(b, m, deterministic)
}
func (m *GoGoProtoStdTime) XXX_Merge(src proto.Message) {
	xxx_messageInfo_GoGoProtoStdTime.Merge(m, src)
}
func (m *GoGoProtoStdTime) XXX_Size() int {
	return xxx_messageInfo_GoGoProtoStdTime.Size(m)
}
func (m *GoGoProtoStdTime) XXX_DiscardUnknown() {
	xxx_messageInfo_GoGoProtoStdTime.DiscardUnknown(m)
}

var xxx_messageInfo_GoGoProtoStdTime proto.InternalMessageInfo

func (m *GoGoProtoStdTime) GetTimestamp() *time.Time {
	if m != nil {
		return m.Timestamp
	}
	return nil
}

func init() {
	proto.RegisterType((*Metadata)(nil), "mwitkow.testproto.Metadata")
	proto.RegisterType((*PingId)(nil), "mwitkow.testproto.PingId")
	proto.RegisterType((*Ping)(nil), "mwitkow.testproto.Ping")
	proto.RegisterType((*PingRequest)(nil), "mwitkow.testproto.PingRequest")
	proto.RegisterType((*Pong)(nil), "mwitkow.testproto.Pong")
	proto.RegisterType((*OneOfLogField)(nil), "mwitkow.testproto.OneOfLogField")
	proto.RegisterType((*PongRequest)(nil), "mwitkow.testproto.PongRequest")
	proto.RegisterType((*GoGoProtoStdTime)(nil), "mwitkow.testproto.GoGoProtoStdTime")
}

func init() { proto.RegisterFile("fields.proto", fileDescriptor_d39ad626ec0e575e) }

var fileDescriptor_d39ad626ec0e575e = []byte{
	// 427 bytes of a gzipped FileDescriptorProto
	0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0x9c, 0x92, 0xcd, 0x8e, 0xd3, 0x30,
	0x14, 0x85, 0x27, 0x25, 0xad, 0xe8, 0xed, 0x20, 0x81, 0xf9, 0x99, 0x4e, 0x91, 0x48, 0xe5, 0x0d,
	0x45, 0x68, 0x1c, 0x31, 0xac, 0x60, 0xc1, 0x22, 0x0b, 0x4a, 0x25, 0xd0, 0x54, 0x66, 0xf6, 0x95,
	0x83, 0x5d, 0x63, 0x4d, 0x93, 0x5b, 0x1a, 0x87, 0x91, 0xba, 0xe1, 0x15, 0x58, 0xf2, 0x74, 0x65,
	0xc1, 0x1b, 0xf4, 0x09, 0x90, 0x9d, 0xfe, 0x04, 0x0d, 0x65, 0xc1, 0x2e, 0xf6, 0x3d, 0xdf, 0x71,
	0xce, 0xd1, 0x85, 0xe3, 0xa9, 0x51, 0x33, 0x59, 0xb0, 0xf9, 0x02, 0x2d, 0x92, 0x7b, 0xd9, 0xb5,
	0xb1, 0x57, 0x78, 0xcd, 0xac, 0x2a, 0xac, 0xbf, 0xea, 0x9d, 0x69, 0x63, 0x3f, 0x97, 0x29, 0xfb,
	0x84, 0x59, 0xac, 0x51, 0x63, 0xec, 0xaf, 0xd3, 0x72, 0xea, 0x4f, 0xfe, 0xe0, 0xbf, 0x2a, 0x87,
	0x5e, 0xa4, 0x11, 0xf5, 0x4c, 0xed, 0x55, 0xd6, 0x64, 0xaa, 0xb0, 0x22, 0x9b, 0x57, 0x02, 0xfa,
	0x0a, 0x6e, 0x7f, 0x50, 0x56, 0x48, 0x61, 0x05, 0x39, 0x83, 0xd0, 0x0a, 0x5d, 0x74, 0x83, 0xfe,
	0xad, 0x41, 0x3b, 0x39, 0x5d, 0xaf, 0xa2, 0x87, 0x33, 0xd4, 0x13, 0xff, 0x4b, 0xaf, 0x69, 0xa6,
	0xac, 0x98, 0xb8, 0x39, 0xe5, 0x5e, 0x46, 0x5f, 0x40, 0x6b, 0x6c, 0x72, 0x3d, 0x92, 0xe4, 0x29,
	0x34, 0x8c, 0xec, 0x06, 0xfd, 0x60, 0xd0, 0x4c, 0x4e, 0xd6, 0xab, 0xe8, 0x7e, 0x0d, 0x9b, 0x9b,
	0x5c, 0x4f, 0x8c, 0xa4, 0xbc, 0x61, 0x24, 0x1d, 0x42, 0xe8, 0x10, 0xf2, 0x6c, 0x07, 0x74, 0xce,
	0x4f, 0xd9, 0x8d, 0x94, 0xac, 0xf2, 0x75, 0x08, 0x79, 0x00, 0xcd, 0xaf, 0x62, 0x56, 0xaa, 0x6e,
	0xa3, 0x1f, 0x0c, 0xda, 0xbc, 0x3a, 0xd0, 0x2b, 0xe8, 0x38, 0x0d, 0x57, 0x5f, 0x4a, 0x55, 0x58,
	0xf2, 0x1c, 0x42, 0xf7, 0xce, 0xc6, 0xf1, 0xe4, 0x80, 0x23, 0xf7, 0x22, 0x12, 0x43, 0xe8, 0xb2,
	0x78, 0xc3, 0xce, 0xf9, 0xe3, 0xbf, 0x88, 0xb7, 0x8d, 0x70, 0x2f, 0xa4, 0x31, 0x84, 0x63, 0xcc,
	0x75, 0x2d, 0x66, 0xfb, 0x66, 0x4c, 0xac, 0xc5, 0xfc, 0x06, 0x77, 0x2e, 0x72, 0x75, 0x31, 0x7d,
	0x8f, 0xfa, 0xad, 0x1b, 0x93, 0x18, 0x5a, 0xa9, 0x58, 0x4c, 0x76, 0xf4, 0xa3, 0xf5, 0x2a, 0x22,
	0x35, 0xba, 0x1a, 0xd2, 0x77, 0x47, 0xbc, 0x99, 0x8a, 0xc5, 0x68, 0x03, 0x2c, 0x1d, 0xd0, 0x38,
	0x00, 0x2c, 0xf7, 0xc0, 0x72, 0x24, 0x93, 0x63, 0x00, 0x23, 0x55, 0x6e, 0xcd, 0xd4, 0xa8, 0x85,
	0xaf, 0x07, 0xff, 0xac, 0x07, 0xff, 0x5d, 0x0f, 0xfa, 0x7a, 0xf0, 0x7f, 0xea, 0xe1, 0x70, 0x77,
	0x88, 0x43, 0x1c, 0xbb, 0xd9, 0x47, 0x2b, 0x2f, 0x4d, 0xa6, 0xc8, 0x1b, 0x68, 0xef, 0x36, 0x6d,
	0xf3, 0x6c, 0x8f, 0x55, 0xbb, 0xc8, 0xb6, 0xbb, 0xc8, 0x2e, 0xb7, 0x8a, 0x24, 0xfc, 0xfe, 0x33,
	0x0a, 0xf8, 0x1e, 0x49, 0xc2, 0x1f, 0xbf, 0x9e, 0x1c, 0xa5, 0x2d, 0x2f, 0x7d, 0xf9, 0x3b, 0x00,
	0x00, 0xff, 0xff, 0xa5, 0x38, 0xdf, 0x28, 0x16, 0x03, 0x00, 0x00,
}
